/*****************************************************************************
*                     Copyright (c)  Gemplus Developpement 1992 - 1997
*
* Name        : ReaderType.h
*
* Description :
*
  Author      : Jean-Luc GIRAUD
  Compiler    : Visual C++, C (Linux)
  Host        : Windows 9x,NT, Linux

* $Id: ReaderType.h,v 1.4 2000/09/19 13:22:45 rousseau Exp $
* $Log: ReaderType.h,v $
* Revision 1.4  2000/09/19 13:22:45  rousseau
* changed ATR_BUF_SIZE to 600
*
* Revision 1.3  2000/09/18 15:17:06  lvictor
* First modifications to support the recent changes made by L.Rousseau to the lib.
* (L.Gauteron + L.Victor)
*
* Revision 1.2  2000/09/04 08:19:04  rousseau
* code cleanup, changed the API and name for [Init|Close]Serial
*
*
******************************************************************************
*
* Warning     :
*
* Remarks     :
*
*****************************************************************************/

#ifndef _READERTYPE_H
#define _READERTYPE_H

#ifndef WIN32 // Linux
	#include <termios.h>
	#define BAUDRATE B9600
	#define TYPE_HANDLE int
#else // Win32
	#include <windows.h>
	#define BAUDRATE CBR_9600
	#define TYPE_HANDLE HANDLE
#endif

/* Reader types for manual selection */
typedef enum READER_TYPE
{
	TLP=0,
	RFTLP,
	UNKNOWN
} rd_type;


/* Reader error codes */
typedef enum RD_ERRORS
{
	RD_NO_ERROR				= 0,
	RD_UNKNOWN_COMMAND,
	RD_DRIVER_LIMITATION,
	RD_WRONG_ARG_LENGTH,
	RD_PROTOCOL_ERROR,
	RD_WRONG_ATR,			/* 5 */
	RD_WRONG_RD_INS,
	RD_MSG_TOO_LONG,
	RD_READ_ERROR,
	RD_CARD_OFF,
	RD_PROG_VOLT_ERROR,		/* 10 */
	RD_UNKNOWN_PROTOCOL,
	RD_LENGTH_ERROR,
	RD_WRONG_NB_OF_PARAM,
	RD_UNKNOWN_CARD_PROTOCOL,
	RD_CARD_MUTE,			/* 15 */
	RD_CARD_PARITY_ERROR,
	RD_CARD_WRONG_PROCEDURE_BYTE,
	RD_TRANSFERT_TO_CARD_INTERRUPTED,
	RD_CARD_REMOVED,
	RD_SHORT_CIRCUIT,		/* 20 */
	RD_CARD_ABSENT,
	/* read on serial port failed */
	RD_READ_FAILED,
	/* reader complained about a transmission error */
	RD_TRANS_ERROR,
	/* write to serial port failed */
	RD_WRITE_FAILED,
	/* Response form reader exceeded buffer length */
	RD_RESP_TOO_LONG,		/* 25 */
	RD_READER_MUTE,
	RD_STATUS_TOO_LONG,
	RD_USELESS_ERROR,
	RD_CHAINING_ERROR,
	RD_RESP_TRUNCATED,		/* 30 */
	RD_UNKNOWN_ERROR,

	RD_NUM_ERRORS
} t_rd_error;


/* No need to look into the details of this structure, */
/* detailed information is only useful for the library */
typedef struct RD
{
	TYPE_HANDLE fd;					/* Device file descriptor */
	rd_type type;					/* reader type */
#ifndef WIN32 // Linux
	struct termios oldtio;			/* port initial state (restored on exit) */
#else // Win32
	DCB oldtio;
#endif
	t_rd_error (*TrError) (int rd_error);	/* Error code translation */
	int (*ResetCardAscii)(TYPE_HANDLE fd, char* atr, int length);
	int (*ResetCardBinary)(TYPE_HANDLE fd, unsigned char* atr, int length);
	int (*PowerDown)(TYPE_HANDLE fd);
	int (*SendIncomingAPDUBinary)(TYPE_HANDLE fd,
				unsigned char CLA,
				unsigned char INS,
				unsigned char P1,
				unsigned char P2,
				unsigned char L,
				const unsigned char *data_in,
				unsigned char *ad_SW1,
				unsigned char *ad_SW2);

	int (*SendOutgoingAPDUBinary)(TYPE_HANDLE fd,
				unsigned char CLA,
				unsigned char INS,
				unsigned char P1,
				unsigned char P2,
				unsigned char L,
				unsigned char *data_out,
				unsigned char *ad_SW1,
				unsigned char *ad_SW2);

	int (*SendIncomingAPDUAscii)(TYPE_HANDLE fd, const char* APDU_Data,
				char* ad_response, int resp_length);

	int (*SendOutgoingAPDUAscii)(TYPE_HANDLE fd, const char* APDU_Data,
				char* ad_response, int resp_length);

	int (*SendIncomingOutgoingAPDUBinary)(TYPE_HANDLE fd,
					unsigned char CLA,
					unsigned char INS,
					unsigned char P1,
					unsigned char P2,
					unsigned char L_in,
					const unsigned char *data_in,
					unsigned char L_out,
					unsigned char *data_out,
					unsigned char *ad_SW1,
					unsigned char *ad_SW2);
	int (*SendIncomingOutgoingAPDUAscii)(TYPE_HANDLE fd, const char* APDU_Data,
					char* ad_response, int resp_length);
} ReaderDesc;

#define ATR_BUF_SIZE 600
#define ISO7816_ERROR_TEXT_SIZE 200

#endif  _READERTYPE_H

