/*****************************************************************************
* Copyright (c)  Gemplus Developpement 1992 - 1997
*
* Name        : READER.H
*
* Description :
*
  Author      : Jean-Luc GIRAUD
  Compiler    : Visual C++, C (Linux)
  Host        : Windows 9x,NT, Linux

* $Id: reader.h,v 1.7 2000/09/19 10:30:21 rousseau Exp $
* $Log: reader.h,v $
* Revision 1.7  2000/09/19 10:30:21  rousseau
* ISO-7816_ErrorStr now adds the SW in hexa
*
* Revision 1.6  2000/09/18 15:17:06  lvictor
* First modifications to support the recent changes made by L.Rousseau to the lib.
* (L.Gauteron + L.Victor)
*
* Revision 1.5  2000/09/15 16:13:51  rousseau
* moved files from common/ to src/
*
* Revision 1.4  2000/09/04 15:29:07  giraud
* Change prototypes for error code handling functions
*
* Revision 1.3  2000/09/04 08:19:04  rousseau
* code cleanup, changed the API and name for [Init|Close]Serial
*
*
******************************************************************************
*
*Warning     :
*
*Remarks     :
*
*****************************************************************************/


#ifndef _READER_H
#define _READER_H

#include "ReaderType.h"

#ifdef __cplusplus
extern "C" {
#endif

/****************************************************************************/
/*                                                                          */
/*  Get the readerlib error message number                                  */
/*                                                                          */
/*  RETURN VALUE: the error number                                          */
/*                                                                          */
/****************************************************************************/
t_rd_error GetRerrno(void);


/****************************************************************************/
/*                                                                          */
/*  Get the readerlib error message text correponding to nErr               */
/*  You can use GetRerrStr(GetRerrno()) to directly get the string          */
/*                                                                          */
/*  RETURN VALUE: a pointer to the error string                             */
/*                                                                          */
/****************************************************************************/
char *GetRerrStr (t_rd_error nErr);


/****************************************************************************/
/*                                                                          */
/*  Get the ISO 7816 error message corresponding to SW1 and SW2             */ 
/*                                                                          */
/*  RETURN VALUE: a pointer to the error string                             */
/*                                                                          */
/****************************************************************************/
char *ISO7816_ErrorStr(unsigned char sw1, unsigned char sw2);


/****************************************************************************/
/*                                                                          */
/*  First command to start communication                                    */
/*                                                                          */
/*  ad_rd       : address of a reader descriptor structure. It should be    */
/*                  allocated by the caller                                 */
/*                                                                          */
/*  device      : name of the communication port ("/dev/ttyS0")             */
/*                                                                          */
/*  type        : type of reader (TLP, RFTLP)                               */
/*                                                                          */
/*  RETURN VALUE: 0 on success                                              */
/*  -1 on error (rerrno is set accordingly)                                 */
/*                                                                          */
/*****************************************************************************/
int InitReader(ReaderDesc *ad_rd, rd_type type, char *device);


/****************************************************************************/
/*                                                                          */
/*  Last command, to close the communication and restore the serial port    */
/*  configuration                                                           */
/*                                                                          */
/*  ad_rd       : address of the reader descriptor structure initialized    */
/*                  in InitReader                                           */
/*                                                                          */
/****************************************************************************/
void CloseReader(ReaderDesc *ad_rd);


/****************************************************************************/
/*                                                                          */
/*  Power down the card. A reset will be needed to send commands again      */
/*                                                                          */
/*  ad_rd       : address of the reader descriptor structure initialized    */
/*                  in InitReader                                           */
/*                                                                          */
/*  RETURN VALUE: 0 on success                                              */
/*  -1 on error (rerrno is set accordingly)                                 */
/*                                                                          */
/****************************************************************************/
int PowerDown(ReaderDesc *ad_rd);


/****************************************************************************/
/*                                                                          */
/*  Reset the card and get the ATR                                          */
/*                                                                          */
/*  ad_rd       : address of the reader descriptor structure initialized    */
/*                  in InitReader                                           */
/*                                                                          */
/*  atr         : string to store the value of the ATR                      */
/*                                                                          */
/*  length      : maximum size of the atr string including '\0' (in char)   */
/*                                                                          */
/*  RETURN VALUE: actual length of the ATR including '\0' (in char)         */
/*  -1 on error (rerrno is set accordingly)                                 */
/*                                                                          */
/*****************************************************************************/
int ResetCardAscii(ReaderDesc *ad_rd, char*atr, int length);


/****************************************************************************/
/*                                                                          */
/*  Reset the card and get the ATR                                          */
/*                                                                          */
/*  ad_rd       : address of the reader descriptor structure initialized    */
/*                  in InitReader                                           */
/*                                                                          */
/*  atr         : array of bytes to store the value of the ATR              */
/*                                                                          */
/*  length      : maximum size of the atr array (in byte)                   */
/*                                                                          */
/*  RETURN VALUE: actual length of the ATR (in byte)                        */
/*  -1 on error (rerrno is set accordingly)                                 */
/*                                                                          */
/****************************************************************************/
int ResetCardBinary(ReaderDesc *ad_rd, unsigned char*atr, int length);


/****************************************************************************/
/*                                                                          */
/*  Used to send an incoming command to the card (ie data are send to       */
/*  the card).                                                              */
/*                                                                          */
/*  ad_rd       : address of the reader descriptor structure initialized    */
/*                  in InitReader                                           */
/*                                                                          */
/*  APDU_Data   : APDU header & data string (ex: "00A40000023F00")          */
/*                              CLA  _____________| | | | | |               */
/*                              INS  _______________| | | | |               */
/*                              P1   _________________| | | |               */
/*                              P2   ___________________| | |               */
/*                              Lc   _____________________| |               */
/*                              DATA _______________________|               */
/*                                                                          */
/*  ad_response : string to hold the response of the card (ex: "9000")      */
/*                                                                          */
/*  resp_length : maximum size of response string including '\0' (in char)  */
/*                                                                          */
/*  RETURN VALUE: actual length of the response including '\0' (in char)    */
/*  -1 on error (rerrno is set accordingly)                                 */
/*                                                                          */
/****************************************************************************/
int SendIncomingAPDUAscii(ReaderDesc *ad_rd, const char*APDU_Data,
    char*ad_response, int resp_length);

/****************************************************************************/
/*                                                                          */
/*  Used to send an outgoing command to the card (ie data are send by       */
/*  the card).                                                              */
/*                                                                          */
/*  ad_rd       : address of the reader descriptor structure initialized    */
/*                  in InitReader                                           */
/*                                                                          */
/*  APDU_Data   : APDU header & data string (ex: "00C0000012")              */
/*                              CLA  _____________| | | | |                 */
/*                              INS  _______________| | | |                 */
/*                              P1   _________________| | |                 */
/*                              P2   ___________________| |                 */
/*                              Le   _____________________|                 */
/*                                                                          */
/*  ad_response : string to hold the response of the card (ex: "??????9000")*/
/*  The SW is included as part of the answer                                */
/*                                                                          */
/*  resp_length : maximum size of response string including '\0' (in char)  */
/*                                                                          */
/*  RETURN VALUE: actual length of the response including '\0' (in char)    */
/*  -1 on error (rerrno is set accordingly)                                 */
/*                                                                          */
/****************************************************************************/
int SendOutgoingAPDUAscii(ReaderDesc *ad_rd,  const char*APDU_Data,
    char*ad_response, int resp_length);

/****************************************************************************/
/*                                                                          */
/*  Used to send an incoming command to the card (ie data are send to       */
/*  the card).                                                              */
/*                                                                          */
/*  ad_rd       : address of the reader descriptor structure initialized    */
/*                  in InitReader                                           */
/*                                                                          */
/*  CLA         : APDU class byte                                           */
/*                                                                          */
/*  INS         : APDU ins byte                                             */
/*                                                                          */
/*  P1          : APDU P1 byte                                              */
/*                                                                          */
/*  P2          : APDU P2 byte                                              */
/*                                                                          */
/*  L           : APDU given length (ie length of data sent to the card)    */
/*                                                                          */
/*  data_in     : array of bytes holding the data to send to the card       */
/*                                                                          */
/*  ad_SW1      : address of a byte to store the first half of the SW       */
/*                                                                          */
/*  ad_SW2      : address of a byte to store the second half of the SW      */
/*                                                                          */
/*  RETURN VALUE: actual length of the card response (in byte)              */
/*  -1 on error (rerrno is set accordingly)                                 */
/*                                                                          */
/****************************************************************************/
int SendIncomingAPDUBinary(struct RD *rd,
    unsigned char CLA,
    unsigned char INS,
    unsigned char P1,
    unsigned char P2,
    unsigned char L,
    const unsigned char *data_in,
    unsigned char *ad_SW1,
    unsigned char *ad_SW2);


/****************************************************************************/
/*                                                                          */
/*  Used to send an outgoing command to the card (ie data are send by       */
/*  the card).                                                              */
/*                                                                          */
/*  ad_rd       : address of the reader descriptor structure initialized    */
/*                  in InitReader                                           */
/*                                                                          */
/*  CLA         : APDU class byte                                           */
/*                                                                          */
/*  INS         : APDU ins byte                                             */
/*                                                                          */
/*  P1          : APDU P1 byte                                              */
/*                                                                          */
/*  P2          : APDU P2 byte                                              */
/*                                                                          */
/*  L           : APDU expected length (ie length of data sent by the card) */
/*                                                                          */
/*  data_out    : array of bytes to hold the data sent by the card          */
/*  The SW is NOT included as part of the answer                            */
/*  data_out is assumed to be at least L byte long                          */
/*                                                                          */
/*  ad_SW1      : address of a byte to store the first half of the SW       */
/*                                                                          */
/*  ad_SW2      : address of a byte to store the second half of the SW      */
/*                                                                          */
/*  RETURN VALUE: actual length of the card response (in byte)              */
/*  -1 on error (rerrno is set accordingly)                                 */
/*                                                                          */
/****************************************************************************/
int SendOutgoingAPDUBinary(struct RD *rd,
    unsigned char CLA,
    unsigned char INS,
    unsigned char P1,
    unsigned char P2,
    unsigned char L,
    unsigned char *data_out,
    unsigned char *ad_SW1,
    unsigned char *ad_SW2);


/****************************************************************************/
/*                                                                          */
/*  Used to send an incoming outgoing command to the card (ie data are send */
/*  to & by he card).                                                       */
/*                                                                          */
/*  ad_rd       : address of the reader descriptor structure initialized    */
/*                  in InitReader                                           */
/*                                                                          */
/*  CLA         : APDU class byte                                           */
/*                                                                          */
/*  INS         : APDU ins byte                                             */
/*                                                                          */
/*  P1          : APDU P1 byte                                              */
/*                                                                          */
/*  P2          : APDU P2 byte                                              */
/*                                                                          */
/*  L_in         : APDU given length (ie length of data sent to the card)   */
/*                                                                          */
/*  data_in     : array of bytes holding the data to send to the card       */
/*                                                                          */
/*  L_out       : APDU expected length (ie length of data sent by the card) */
/*                                                                          */
/*  data_out    : array of bytes to hold the data sent by the card          */
/*  The SW is NOT included as part of the answer                            */
/*  data_out is assumed to be at least L byte long                          */
/*                                                                          */
/*  ad_SW1      : address of a byte to store the first half of the SW       */
/*                                                                          */
/*  ad_SW2      : address of a byte to store the second half of the SW      */
/*                                                                          */
/*  RETURN VALUE: actual length of the card response (in byte)              */
/*  -1 on error (rerrno is set accordingly)                                 */
/*                                                                          */
/*****************************************************************************/
int SendIncomingOutgoingAPDUBinary(ReaderDesc *ad_rd,
    unsigned char CLA,
    unsigned char INS,
    unsigned char P1,
    unsigned char P2,
    unsigned char L_in,
    unsigned char *data_in,
    unsigned char L_out,
    unsigned char *data_out,
    unsigned char *ad_SW1,
    unsigned char *ad_SW2);


/****************************************************************************/
/*                                                                          */
/*  Used to send an incoming outgoing command to the card (ie data are      */
/*  send to & by the card).                                                 */
/*                                                                          */
/*  ad_rd       : address of the reader descriptor structure initialized    */
/*                  in InitReader                                           */
/*                                                                          */
/*  APDU_Data   : APDU header & data string (ex: "00A40000023F00")          */
/*                              CLA  _____________| | | | | |               */
/*                              INS  _______________| | | | |               */
/*                              P1   _________________| | | |               */
/*                              P2   ___________________| | |               */
/*                              Lc   _____________________| |               */
/*                              DATA _______________________|               */
/*                                                                          */
/*  ad_response : string to hold the response of the card (ex: "??????9000")*/
/*  The SW is included as part of the answer                                */
/*                                                                          */
/*  resp_length : maximum size of response string including '\0' (in char)  */
/*                                                                          */
/*  RETURN VALUE: actual length of the response including '\0' (in char)    */
/*  -1 on error (rerrno is set accordingly)                                 */
/*                                                                          */
/****************************************************************************/
int SendIncomingOutgoingAPDUAscii(ReaderDesc *ad_rd, const char*APDU_Data,
                  char*ad_response, int resp_length);
#ifdef __cplusplus
}
#endif

#endif
